require 'yaml'
require 'haredo/peer'

HAREDO_DEFAULT_CONFIG_FILE = '/etc/haredo/haredo.conf'
HAREDO_DEFAULT_PID_FILE    = '/var/run/haredo.pid'

class Time
  def isoDate()
    return self.strftime("%Y-%m-%d %H:%M:%S")
  end
end

module HareDo
module Config

  def loadConfig(file=nil)
    @config_file = file || $haredo_default_config_file || HAREDO_DEFAULT_CONFIG_FILE    
    @config      = []    
    @config      = YAML.load_file(@config_file)
    @pid_file    = $haredo_default_pid_file || HAREDO_DEFAULT_PID_FILE
  end

  def saveConfig()
    file = File.open(@config_file, 'w')
    file.write(YAML::dump(@config))
    file.close()
  end

  # Connect to the RabbitMQ broker
  #
  # @return Return a client instance if connection was successful, nil otherwise
  def connectBroker()

    account = @config['system']['broker']

    params = {
      :user     => account['user'],
      :password => account['password'],
      :host     => account['host'],
      :port     => account['port'], 
      :vhost    => account['vhost'],
      :ssl      => account['ssl']
    }

    peer = HareDo::Peer.new()
    
    peer.connect(params)

    return peer
  end

end # module Config
end # module HareDo
